@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2015 Lenovo
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,18 +12,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# This represents the server side of a session object
-# Split into a separate file to avoid overly manipulating the as-yet
-# client-centered session object
+"""This represents the server side of a session object
+Split into a separate file to avoid overly manipulating the as-yet
+client-centered session object
+"""
+
 import collections
 import hashlib
 import hmac
 import os
-import pyghmi.ipmi.private.constants as constants
-import pyghmi.ipmi.private.session as ipmisession
+import socket
 import struct
 import uuid
 
+import pyghmi.ipmi.private.constants as constants
+import pyghmi.ipmi.private.session as ipmisession
+
 
 class ServerSession(ipmisession.Session):
     def __new__(cls, authdata, kg, clientaddr, netsocket, request, uuid,
@@ -41,11 +43,11 @@ class ServerSession(ipmisession.Session):
         # role = request[1]
         self.clientsessionid = request[4:8]
         # TODO(jbjohnso): intelligently handle integrity/auth/conf
-        #for now, forcibly do cipher suite 3
+        # for now, forcibly do cipher suite 3
         self.managedsessionid = os.urandom(4)
-        #table 13-17, 1 for now (hmac-sha1), 3 should also be supported
-        #table 13-18, integrity, 1 for now is hmac-sha1-96, 4 is sha256
-        #confidentiality: 1 is aes-cbc-128, the only one
+        # table 13-17, 1 for now (hmac-sha1), 3 should also be supported
+        # table 13-18, integrity, 1 for now is hmac-sha1-96, 4 is sha256
+        # confidentiality: 1 is aes-cbc-128, the only one
         self.privlevel = 4
         response = (bytearray([clienttag, 0, self.privlevel, 0]) +
                     self.clientsessionid + self.managedsessionid +
@@ -60,6 +62,8 @@ class ServerSession(ipmisession.Session):
                  bmc):
         # begin conversation per RMCP+ open session request
         self.uuid = uuid
+        self.currhashlib = hashlib.sha1
+        self.currhashlen = 12
         self.rqaddr = constants.IPMI_BMC_ADDRESS
         self.authdata = authdata
         self.servermode = True
@@ -68,6 +72,7 @@ class ServerSession(ipmisession.Session):
         self.sessionid = 0
         self.bmc = bmc
         self.lastpayload = None
+        self.rqlun = None  # This will be provided by the client
         self.broken = False
         self.authtype = 6
         self.integrityalgo = 0
@@ -75,8 +80,12 @@ class ServerSession(ipmisession.Session):
         self.kg = kg
         self.socket = netsocket
         self.sockaddr = clientaddr
+        self.pendingpayloads = collections.deque([])
         self.pktqueue = collections.deque([])
-        ipmisession.Session.bmc_handlers[clientaddr] = self
+        if clientaddr not in ipmisession.Session.bmc_handlers:
+            ipmisession.Session.bmc_handlers[clientaddr] = {bmc.port: self}
+        else:
+            ipmisession.Session.bmc_handlers[clientaddr][bmc.port] = self
         response = self.create_open_session_response(bytearray(request))
         self.send_payload(response,
                           constants.payload_types['rmcpplusopenresponse'],
@@ -96,7 +105,7 @@ class ServerSession(ipmisession.Session):
         self.maxpriv = self.rolem & 0b111
         namepresent = data[27]
         if namepresent == 0:
-            #ignore null username for now
+            # ignore null username for now
             return
         self.username = bytes(data[28:])
         if self.username.decode('utf-8') not in self.authdata:
@@ -151,7 +160,7 @@ class ServerSession(ipmisession.Session):
                                     ).digest()
         authcode = struct.pack("%dB" % len(data[8:]), *data[8:])
         if expectedauthcode != authcode:
-            #TODO(jjohnson2): RMCP error back at invalid rakp3
+            # TODO(jjohnson2): RMCP error back at invalid rakp3
             return
         clienttag = data[0]
         if data[1] != 0:
@@ -206,7 +215,7 @@ class ServerSession(ipmisession.Session):
         After the session inactivity timeout, this invalidate the client
         session.
         """
-        #for now, we will have a non-configurable 60 second timeout
+        # for now, we will have a non-configurable 60 second timeout
         pass
 
     def _handle_channel_auth_cap(self, request):
@@ -225,9 +234,9 @@ class ServerSession(ipmisession.Session):
 
 
 class IpmiServer(object):
-    #auth capabilities for now is a static payload
-    #for now always completion code 0, otherwise ignore
-    #authentication type fixed to ipmi2, ipmi1 forbidden
+    # auth capabilities for now is a static payload
+    # for now always completion code 0, otherwise ignore
+    # authentication type fixed to ipmi2, ipmi1 forbidden
     # 0b10000000
 
     def __init__(self, authdata, port=623, bmcuuid=None, address='::'):
@@ -239,7 +248,7 @@ class IpmiServer(object):
                         reasonable subset.
         :param port: The default port number to bind to.  Defaults to the
                      standard 623
-        :param address: The IPv6 address to bind to. Defaults to '::' (all
+        :param address: The IP address to bind to. Defaults to '::' (all
                         zeroes)
         """
         self.revision = 0
@@ -265,9 +274,11 @@ class IpmiServer(object):
                                    authstatus, chancap, *oemdata)
         self.kg = None
         self.timeout = 60
-        self.serversocket = ipmisession.Session._assignsocket(
-            (address, port, 0, 0))
-        ipmisession.Session.bmc_handlers[self.serversocket] = self
+        self.port = port
+        addrinfo = socket.getaddrinfo(address, port, 0,
+                                      socket.SOCK_DGRAM)[0]
+        self.serversocket = ipmisession.Session._assignsocket(addrinfo)
+        ipmisession.Session.bmc_handlers[self.serversocket] = {0: self}
 
     def send_auth_cap(self, myaddr, mylun, clientaddr, clientlun, clientseq,
                       sockaddr):
@@ -310,7 +321,9 @@ class IpmiServer(object):
                               self.serversocket, data[16:], self.uuid,
                               bmc=self)
                 return
-            data = data[13:]  # ditch 13 bytes so the payload works out
+            # ditch two byte, because ipmi2 header is two
+            # bytes longer than ipmi1 (payload type added, payload length 2).
+            data = data[2:]
         myaddr, netfnlun = struct.unpack('2B', bytes(data[14:16]))
         netfn = (netfnlun & 0b11111100) >> 2
         mylun = netfnlun & 0b11
